#!/usr/bin/perl

#use strict;
#use warnings;

###################################################################
#                                                                 #
# (C) COPYRIGHT International Business Machines Corp. 1985, 1989  #
# All Rights Reserved                                             #
# Licensed Materials - Property of IBM                            #
#                                                                 #
# This is an IBM Internal Tool.                                   #
# Use at your own risk.                                           #
###################################################################

###################################################################
# ftpgen.pl - run ftp jobs in parallel.
#
#   File List format: fullpath_filename Target_directory
#   Ex: /tmp/foobar.sh /u01/jubal/incoming
###################################################################

###################################################################
#  HISTORY
#  Version:    # Date:     Description
#  =======     # =======   ===========================================
my $VV="1.00"; # 2/17/17   Initial version
###################################################################

#----------------------------------------------------------------
#-- INITIALIZATON  ----------------------------------------------
#----------------------------------------------------------------

use Getopt::Long;
use File::Basename;

my $cnt=0;

my $dbg;
my $version;
my $ignwarn=0;
my $VER = $VV;
my $CNTE=0; 
my $CNTF=0; 
my $CNTW=0; 
my $CNTS=0; 
my $CNTX=0;
my $ECNT=0;
my $FCNT=0;
my $WCNT=0;
my $SCNT=0;
my $XCNT=0;
my $SET;
my $fnd;
my $val;


#----------------------------------------------------------------
#--  START OPTION HANDLING  -------------------------------------
#----------------------------------------------------------------
sub do_opts {
  my $man  = 0;
  my $help = 0;
  my $retval;

  use Getopt::Long;
  $retval=GetOptions( 
            "iw"           =>\$ignwarn,
            "stats=s"      =>\$statnum,
            "dbg"          =>\$dbg,
            "version"      =>\$version,
            'help|?'       =>\$help,
            'man'          =>\$man
            ) ;

#  # handles unknown options. (ARGV isnt empty!)
#  if ( $ARGV[0] ) {
#    print "Unprocessed by Getopt::Long\n";
#    foreach (@ARGV) { print "$_\n"; }
#    pod2usage(-message => "Unprocessed by Getopt::Long",
#              -exitval => 2,
#              -verbose => 0,
#              -output  => \*STDERR);
#  }

  #- Parse options and print usage if there is a syntax error,
  #- or if usage was explicitly requested.
  pod2usage(1)             if $help;
  pod2usage(-verbose => 2) if $man;

 #- If no arguments were given, then allow STDIN to be used only
 #- if it's not connected to a terminal (otherwise print usage)
   pod2usage("$0: No files given.")  if ((@ARGV == 0) && (-t STDIN));

  return $retval;
}
#####################################################
# CUTE TRICK. Allow opts to be specified via env var.
#####################################################
#my $getopts;
#
#$getopts=$ENV{'GETERR_OPTS'};
#if ( $getopts ne "" ) { 
#  @ARGV = split(/ /,$getopts); 
#}
&do_opts or pod2usage(1) or die "do_opts failed\n" ;
if ( defined($version) )  { die "$0\tVersion: $VER\n"; }
#----------------------------------------------------------------
#--  STOP  OPTION HANDLING  -------------------------------------
#----------------------------------------------------------------

# check for errors
$cnt=0;
$CNTE=0; $CNTF=0; $CNTW=0; $CNTS=0; $CNTX=0;
$SET=4;
$fnd=0;

while ( <> ) {
  next if  ( /Error: 79:/ ) ;
  next if  ( /TNS-00511/  ) ;
  next if  ( /TNS-12541/  ) ;
  next if  ( /TNS-12560/  ) ;

  if  ( /Error/i )      { $ECNT+=1; }
  if  ( /fatal/i )      { $FCNT+=1; }
  if  ( /warning/i )    { $WCNT+=1; }
  if  ( /severe/i )     { $SCNT+=1; }
  if  ( /exception/i )  { $XCNT+=1; }
  if  ( /exiting with status (\d+)/ ) {
    $cnt=$cnt+1;
    $val=$1;
    if ($val != 0 ) {
      print("ERROR FOUND: $_");
      $fnd=1;
    }
  }
}

if ( defined($statnum) ) { $SET=$statnum; }

if ( ($cnt % $SET) != 0 ) {
  print("MISSING STATUS LINES: Only FOUND: $cnt\n");
  print("Count should be $SET...\n");
  $fnd=1;
}

if ($ECNT != $CNTE ) { print("ERRORS:     Count is: $ECNT, should be $CNTE\n"); $fnd=1;}
if ($BCNT != $CNTF ) { print("FATALS:     Count is: $FCNT, should be $CNTF\n"); $fnd=1;}
if ($CCNT != $CNTW ) { print("WARNINGS:   Count is: $WCNT, should be $CNTW\n"); $fnd=1;}
if ($SCNT != $CNTS ) { print("SERVERE:    Count is: $SCNT, should be $CNTS\n"); $fnd=1;}
if ($XCNT != $CNTX ) { print("EXCECTIONS: Count is: $XCNT, should be $CNTX\n"); $fnd=1;}

if ( $fnd == 0 )    { print("OUTPUT FILE APPEARS CLEAN. No errors found!\n"); }
if ( $fnd != 0 )    { print("\n\n\tPROBLEMS FOUND. PLEASE CHECK FILES...\n\n"); }
