#!/usr/bin/perl

#use strict;
#use warnings;

###################################################################
#                                                                 #
# (C) COPYRIGHT International Business Machines Corp. 1985, 1989  #
# All Rights Reserved                                             #
# Licensed Materials - Property of IBM                            #
#                                                                 #
# This is an IBM Internal Tool.                                   #
# Use at your own risk.                                           #
###################################################################

###################################################################
# ftpgen.pl - run ftp jobs in parallel.
#
#   File List format: fullpath_filename Target_directory
#   Ex: /tmp/foobar.sh /u01/jubal/incoming
###################################################################

###################################################################
#   HISTORY
#   Version:    #  Date:      Description
#   =======     #  ========   =======================================
#my $VV="1.00"; #  02/17/17   Initial version
#my $VV="1.01"; #  02/17/17   support 12.2.5 and 12.2.6 of EBS
#my $VV="1.02"; #  11/21/17   Fixed statements on up and down
 my $VV="1.03"; #  01/17/18   Changed how dbms up is determined
###################################################################

#----------------------------------------------------------------
#-- INITIALIZATON  ----------------------------------------------
#----------------------------------------------------------------

use Getopt::Long;
use File::Basename;


my $version=0;
my $ignwarn=0;
my $VER = $VV;
my $cmd;
my $dbms=0;
my $apps=0;
my $other=0;
my $nolist=0;
my %dbmsprocs;
my %appprocs;
my %othprocs;
my $dbmslist;
my $applist;
my $tmp1;
my $rest;
my $found;
my $tot=0;
my $inps='';
my $first_line='';
my $usr;
my $pid;
my $ppid;
my $ct;
my $stime;
my $tty;
my $time;
my $usrarg;
my $UPNUM=20;


#----------------------------------------------------------------
#--  START OPTION HANDLING  -------------------------------------
#----------------------------------------------------------------
sub do_opts {
  my $man  = 0;
  my $help = 0;
  my $retval;

  use Getopt::Long;
  $retval=GetOptions( 
            "f=s"          =>\$inps,
            "u=s"          =>\$usrarg,
            "apps"         =>\$apps,
            "dbms"         =>\$dbms,
            "other"        =>\$other,
            "nolist"       =>\$nolist,
            "pids"         =>\$pids,
            "dbg"          =>\$dbg,
            "version"      =>\$version,
            'help|?'       =>\$help,
            'man'          =>\$man
            ) ;

#  # handles unknown options. (ARGV isnt empty!)
#  if ( $ARGV[0] ) {
#    print "Unprocessed by Getopt::Long\n";
#    foreach (@ARGV) { print "$_\n"; }
#    pod2usage(-message => "Unprocessed by Getopt::Long",
#              -exitval => 2,
#              -verbose => 0,
#              -output  => \*STDERR);
#  }

  #- Parse options and print usage if there is a syntax error,
  #- or if usage was explicitly requested.
  pod2usage(1)             if $help;
  pod2usage(-verbose => 2) if $man;

# #- If no arguments were given, then allow STDIN to be used only
# #- if it's not connected to a terminal (otherwise print usage)
#   pod2usage("$0: No files given.")  if ((@ARGV == 0) && (-t STDIN));

  return $retval;
}
#####################################################
# CUTE TRICK. Allow opts to be specified via env var.
#####################################################
#my $getopts;
#
#$getopts=$ENV{'GETERR_OPTS'};
#if ( $getopts ne "" ) { 
#  @ARGV = split(/ /,$getopts); 
#}
&do_opts or pod2usage(1) or die "do_opts failed\n" ;
if ( $version != 0 )  { die "$0\tVersion: $VER\n"; }
#----------------------------------------------------------------
#--  STOP  OPTION HANDLING  -------------------------------------
#----------------------------------------------------------------

my @dbmslist = (
  "ora_",
  "oracleVIS",
  "tnslsnr VIS",
  "tnslsnr LISTENER",
  );

my @applist = (
  "httpd.conf",
  "rotatelogs",
  "java",
  "opmn",
  "CYQLIB",
  "FFTM",
  "adRegisterWLSListeners.pl",
  "FNDCRM",
  "FNDIMON",
  "FNDLIBR",
  "FNDSM",
  "INCTM",
  "INVLIBR",
  "MRCLIB",
  "PALIBR",
  "POXCON",
  "RCVOLTM",
  "tnslsnr APPS_VIS",
  "httpd.worker",
  );


###########################################
# Functions
###########################################
#
sub runsql {
   $sqlout = `sqlplus -s '/ as sysdba' <<END
   SELECT INSTANCE_NAME, STATUS, DATABASE_STATUS FROM V\\\$INSTANCE;
   quit;
END
`;
}

sub isdbmsup{
  $cnt=0;
  $maxcnt=5;
  $sleept=1;
  
#  while ( $cnt < $maxcnt ) {
    runsql;
    if ( $sqlout =~ /OPEN/ ) { $isup="true"}
    if ( $sqlout =~ /ERROR/ )         { $founderr="true"; }
#    last if ( defined($isup) );
#    sleep($sleept);
#    $cnt++;
#  }
  
  if ( defined($isup) ) {
     #print("DBMS is up and taking connections\n");
     return(0);
  }
  if ( defined($founderr) ) {
#     print("DBMS connectivity is showing errors and is not avaiable\n");
     return(-5);
  }
}

###########################################3
#Main
###########################################3
#

$cmd="ps -aef | fgrep -v aef |";
if ( $inps ne '' ) { $cmd=$inps; }

open(my $fh, "$cmd") or die "Can't open < $cmd $!";

while ( <$fh>  ) {
  #prove its the ps command. First line should be UID header
  #
  if ( ($first_line eq '') ) { $first_line="true"; next; }

   
  ####################################################
  # unpack the line. we'll want to keep pieces...
  #A8       A8       A8       A3  A8      A6      A5    A20
  ($usr,$pid,$ppid,$ct,$stime,$tty,$time,$cmd) = unpack("A8A9A9A4A9A7A6A512",$_);
  if ( defined($dbg) ) {
    print("usr: $usr pid: $pid\n");
    print("ppid: $ppid ct: $ct\n");
    print("stime: $stime tty: $tty time: $time\n");
    print("cmd: $cmd\n");
    print("$_");
  }
  # ignore these lines. we dont care

  next if ( $cmd =~ /defunct/ );
  next if ( ($usrarg ne '') && ($usr !~ $usrarg) ) ;
   

  if ($nolist != 0 ) { print; }
  $found="false";
  foreach my $pat (@dbmslist ) {
    if ( $cmd =~ /$pat/ ) { 
      $dbmsprocs{$pat}++; 
      $found="true";
      next;
    }
  }
  foreach my $pat (@applist ) {
    if ( $cmd =~ /$pat/ ) { 
      $appprocs{$pat}++; 
      $found="true";
      next;
    }
  }
  if ( $found eq "false" ) {
    ($tmp1,$rest) = split(' ',$cmd,2);
#    print("CMD: $cmd TMP1: $tmp1 REST: $rest\n");
    $othprocs{$tmp1}++;
  }
}


if ($dbms != 0 ) {
  $tot=0;
  print("DBMS:=========================\n");
  foreach my $pat (keys %dbmsprocs) {
    printf("%30s: %d\n",$pat,$dbmsprocs{$pat});
    $tot=$tot+$dbmsprocs{$pat};
  } 
  print("DBMS: \tTotal: $tot\n\n");
  
  $retval=isdbmsup;
  if ( $retval == 0 ) {
    print("\n\t\t ***** DBMS is UP *****\n\n");
  } else {
    print("\n\t\t ***** DBMS is DOWN *****\n\n");
  }
}
if ($apps != 0 ) {
  $tot=0;
  print("APPS:=========================\n");
  foreach my $pat (keys %appprocs) {
    printf("APPS: %30s: %d\n",$pat,$appprocs{$pat});
    $tot=$tot+$appprocs{$pat};
  } 
  print("APPS: \tTotal: $tot\n\n");
  if ( $tot > 37 )  {
      print("\n\t\t ***** APPSVR is UP *****\n\n");
  } else {
    if ( $tot > 2 )  {
      print("\n\t\t ***** APPSVR is INDETERMIMANT *****\n\n");
    } else {
      print("\n\t\t ***** APPSVR is DOWN *****\n\n");
    }
  }
}

if ($other != 0 ) {
  $tot=0;
  print("REST:=========================\n");
  foreach my $pat (keys %othprocs) {
    printf("REST: %20s: %d\n",$pat,$othprocs{$pat});
    $tot=$tot+$othprocs{$pat};
  } 
  print("REST: \tTotal: $tot\n\n");
}


#===========================================================
#===========================================================
#===========================================================
#EVERYTHING after __END__ is ignored by perl, below is the
#help info used by pod_usage
#===========================================================

use Pod::Usage;
__END__

=head1 NAME

ftpgen.pl - run multiple ftp jobs based on file list

=head1 SYNOPSIS

ftpgen.pl 
[-help|?] 
[-man] 
[-version] 
[-jobs number] 
[-host networkip] 
[-user username] 
[-passwd userpassword] 
-l list_file

 Options:
   -help            brief help message
   -man             full documentation
   -version         Displays the version number
   -jobs            Number of run scripts to generate
   -host            Identifier that can reach the target host
   -user            ftp user to login as
   -passwd          password for this ftp user
   -l               Required file with list of files within it.

The list_file should contain a list of files. See man
page for description.

=head1 DESCRIPTION

B<This program> will find *_enu*.log files within the directory:
${SIEBEL_ROOT}.${_IBM_ENTERPRISE_NAME}.${_IBM_SERVER_NAME}/log

It produces a summary of errors.  Actual errors can
be seen with the [-view] option, which creates a temporary file.
[CAUTION: -view creates 1 file of all logs. This may be too big.]

By default, no options are needed, and assumes that you are running
as the siebel user. It looks for the following environment variables
to be defined and correct for default execution:

	SIEBEL_ROOT
	_IBM_ENTERPRISE_NAME
	_IBM_SERVER_NAME

If these environment variables are not found, it is still
possible to use the command, by providing the directory
location using the -dir option, or overriding the environment variables

Environment Variables

GETERR_OPTS environment variable can be used to fix the default
options, rather than to add them to the command line. For example,
if you wish each invocation of cerr.pl to use a 50 column width, 
you could set GETERR_OPTS="-c 50"

Ignore File Syntax

The perl variable $IGFILE defines the location of the ignore file.
If the file exists and cerrs.pl is invoked with -ign, then the file
is used to filter which errors to ignore. The format of the IGFILE
is a list of patterns to ignore. For example my $IGFILE might contain
the following patterns:

 Fail to load the SearchServer DLLs
 Please enter a new key to proceed
 SBL-DAT-00171
 SBL-DAT-00260
 SBL-DAT-00523
 SBL-OMS-00203
 SBL-SCB-00015
 SBL-SFS-00118
 SBL-SRH-00155
 SBL-SRH-00169
 SBL-UIF-00335
 SBL-UIF-00401

Any errors matching the above patterns will be ignored upon invocation.

cerrs.pl has a built in internal ignore list. Look for 'iglist' array
within the cerrs.pl file for the list. To diable this, use the flag
'noiglist' upon invocation.

Also cerrs.pl has a substitution list. This list contains pattersn
for removing unique values from logfile lines. Doing this may collapse
a large number of log messages into a single group. The command line
option 'nosubstlist' and 'substfile' parameter relate to this list.

=cut  

