#!/usr/bin/perl

###################################################################
#
#   (C) Copyright IBM Corp. 2003 All rights reserved.
#
# This is an IBM Internal Tool developed to aid in managing
# and examining Siebel Applications.  There is no official IBM
# support.  Use at your own risk. Developed for Siebel 7.7 and v8.0
#
###################################################################


use Getopt::Long;
use    Pod::Usage;


$VER = 'V1.10';
######################################################
# HISTORY
# Vers:   Date:     Description
# v1.00	  7/28/04   Wrote the beast
# v1.10	  1/08/06   Added sub db2_is_up
######################################################


######################################################
# FUNCTIONS
######################################################

########################################################################
# Usage: &db2_is_up("DBMSNAME")
#   Returns 1 if true,  the dbms is up and running
#   Returns 0 if false, the dbms is NOT UP.
#
# EXAMPLE:
#   $dbms="SPERF64";
#   if ( &db2_is_up ($dbms) ) { 
#     print("DBMS: $dbms is running\n");
#   } else {
#     print("DBMS: $dbms is not started\n");
#   }
########################################################################
  
sub db2_is_up {

  $USER=$ENV{'USER'};
  $target=$_[0];

  die ("db2_is_up requires the dbms name as the argument\n")
     if ( !defined($target) ) ;
  
  #$pattern="db2sysc";
  $pattern="db2loggw";
  $retval=0;
  
  if ($USER eq '') { 
    die ("environment variable USER does not exist.\n");
  }
  
  $dbcmd="ps -fu $USER | fgrep $pattern | fgrep -v fgrep | ";
  open (DBF1, $dbcmd) or die "Cant run $dbcmd\n" ;
  # MAIN LOOP
  while ( <DBF1> ) {
   ($f1,$f2,$f3,$f4,$f5,$f6,$f7,$proc,$dbname,$rest)=split;
   if ($proc eq "$pattern") {
       if ($dbname eq "($target)" ) {
         $retval=1;
         last;
       }
   }
  }
  close(DBF1);
  return $retval;
}

#------------------------------------------------------
# Get the OPTIONS
#------------------------------------------------------
sub do_opts {
  my $man = 0, $help = 0;

  use Getopt::Long;
  $retval=GetOptions( 
            "f=s"      =>\$lafile,
            "dbg"      =>\$dbg,
            "version"  =>\$version,
            'help|?'   =>\$help,
            man        =>\$man
            ) ;

  # handles unknown options. (ARGV isnt empty!)
  if ( $ARGV[0] ) {
    print "Unprocessed by Getopt::Long\n";
    foreach (@ARGV) { print "$_\n"; }
    pod2usage(-message => "Unprocessed by Getopt::Long",
              -exitval => 2, -verbose => 0, -output  => \*STDERR);
  }

    ## Parse options and print usage if there is a syntax error,
    ## or if usage was explicitly requested.
  pod2usage(1)             if $help;
  pod2usage(-verbose => 2) if $man;

  # ## If no arguments were given, then allow STDIN to be used only
  # ## if it's not connected to a terminal (otherwise print usage)
  # pod2usage("$0: No files given.")  if ((@ARGV == 0) && (-t STDIN));

  return $retval;
}
&do_opts or pod2usage(1) or die "do_opts failed\n" ;
#----------------------------------------------------------------
#-- end of do_opts  ---------------------------------------------
#----------------------------------------------------------------




$dbmsu=$ENV{"_IBM_DBMSUSER"};
$dbmsp=$ENV{"_IBM_DBMSPSWD"};
$dbmsn=$ENV{"_IBM_DBMSNAME"};

die("env: _IBM_DBMSUSER is not set. fatal\n") if ($dbmsu eq '' ) ;
die("env: _IBM_DBMSPSWD is not set. fatal\n") if ($dbmsp eq '' ) ;
die("env: _IBM_DBMSNAME is not set. fatal\n") if ($dbmsn eq '' ) ;

#if ( ! &db2_is_up ($dbmsn) ) { 
#  die("DBMS: $dbmsn is not started\n");
#}



# start and end connection strings
$constr = "db2 connect to $dbmsn user $dbmsu using $dbmsp" ;
$dbs    = "$constr >/dev/null 2>&1" ;
$dbe    = "db2 terminate >/dev/null 2>&1" ;


$db2cmd = "$dbs ; db2 select \"*\" from syscat.bufferpools; $dbe |";
print("\nInfo: connect command: $constr\n\n");





if (defined($lafile)) { $db2cmd ="$lafile"; }


if (defined($dbg)) {print "OPEN: $db2cmd\n";}
open (DBF, $db2cmd)  or die "Cant run $db2cmd\n" ;

# FORMATS for printf statements
$hfmt="%-15s %-5s %10s %6s %9s\n";
$ofmt="%-15s %-5d %10d %6d %9d\n";

# HEADER OF PRINT STATEMENTS
printf $hfmt, "BPNAME         ","BID","NPAGES","PAGSIZ","SPACE(MB)";
printf $hfmt, "===============","===","======","======","=========";

while ( <DBF> ) {
  if (defined($dbg)) {print "LINE: $_\n";}

  next if ( /---------/ );
  next if ( /^$/ );
  next if ( /selected/ );
  next if ( /ESTORE NUMBLOCKPAGES/ );
  ($bpname,$bufid,$dbpgn,$npages,$pagsiz,
	$estore,$numblkpgs,$blksiz,$ngname) = split;
  $tot=($npages*$pagsiz)/(1024*1024);
  $ttot=$ttot+$tot;
  printf $ofmt, $bpname,$bufid,$npages,$pagsiz,$tot;
}

printf $hfmt, "======","=====","======","======","=====";

$tfmt="%-15s %-5s %10s %6s %9d\n";
printf $tfmt, "TOTAL ", " "," "," ",$ttot;

