#!/usr/bin/perl

###################################################################
#
#   (C) Copyright IBM Corp. 2003 All rights reserved.
#
# This is an IBM Internal Tool developed to aid in managing
# and examining Siebel Applications.  There is no official IBM
# support.  Use at your own risk. Developed for Siebel 7.7 and v8.0
#
###################################################################

use Getopt::Long;
use    Pod::Usage;


######################################################
# Description:
#   This program reads the output of:
#      db2 list tablespaces show detail
#
# HISTORY
# Vers:   Date:         Description
# v1.1    Jun   4 2004  Original version
# v1.2    Jun  14 2004  Making changes. massive
# v1.3    Jul  09 2004  adding -c -y switches
# v1.4    Feb  24 2006  Modified to HOME dir
# v2.1    Feb  25 2006  Created getdb.pl and getdbm.pl
# v2.2    Mar  01 2006  Removed getdb.pl and getbm.pl. smile
# v2.3    Mar  09 2006  Fixed a few bugs
# v2.4    Mar  10 2006  Took out the ability to overrite CFG
######################################################
$VER=2.4;


# Perl special variable for OS
$NTID="Windows_NT";
$os=$^O;

# Look for CFG in local dir
$homedir=$ENV{'HOME'};
if (!defined($homedir)) {
  die("Broken, environment var: HOME is not defined\n");
}
$toolsdir=$ENV{'_IBM_TOOLSDIR'};
if (!defined($toolsdir)) {
  die("Broken, environment var: _IBM_TOOLSDIR is not defined\n");
}
$CFGFILE       = "$homedir/CFG";
$CFGFILEORIG   = "$toolsdir/CFG";


#------------------------------------------------------
# Get the OPTIONS
#------------------------------------------------------
sub do_opts {
  my $man = 0, $help = 0;

  use Getopt::Long;
  $retval=GetOptions( 
        "f=s"     =>\$cmdfile,
        "d=s"     =>\$thisdb,
        "noexec"  =>\$noexec,
        "force"   =>\$force,
        "andflt"  =>\$andflt,
        "c=s"     =>\$comment,
        "y"       =>\$yes_go,

        "v"       =>\$version,       # Show Version
        "dbg"     =>\$dbg,           # Debug info printed too
        'help|?'  =>\$help,          # Show Help
        'man'     =>\$man);          # Show Man page

  # handles unknown options. (ARGV isnt empty!)
  if ( $ARGV[0] ) {
    print "Unprocessed by Getopt::Long\n";
    foreach (@ARGV) { print "$_\n"; }
    pod2usage(-message => "Unprocessed by Getopt::Long",
              -exitval => 2, -verbose => 0, -output  => \*STDERR);
  }

    ## Parse options and print usage if there is a syntax error,
    ## or if usage was explicitly requested.
  pod2usage(1)             if $help;
  pod2usage(-verbose => 2) if $man;

  # ## If no arguments were given, then allow STDIN to be used only
  # ## if it's not connected to a terminal (otherwise print usage)
  #   pod2usage("$0: No files given.")  if ((@ARGV == 0) && (-t STDIN));

  return $retval;
}
#----------------------------------------------------------------
#-- end of do_opts  ---------------------------------------------
#----------------------------------------------------------------

########################################################################
# Usage: &db2_is_up("DBMSNAME")
#   Returns 1 if true, the dbms is up and running
#   Returns 0 if false, the dbms is NOT UP.
#
# EXAMPLE:
#   $dbms="SPERF64";
#   if ( &db2_is_up ($dbms) ) { 
#     print("DBMS: $dbms is running\n");
#   } else {
#     print("DBMS: $dbms is not started\n");
#   }
########################################################################
  
sub db2_is_up {
  local $tmp_dbname;
  $USER=$ENV{'USER'};
  $target=$_[0];

  die ("db2_is_up requires the dbms name as the argument\n")
     if ( !defined($target) ) ;
  
  #$pattern="db2sysc";
  $pattern="db2loggw";
  
  if ($USER eq '') { 
    die ("environment variable USER does not exist.\n");
  }
  
  $dbcmd="ps -fu $USER | fgrep $pattern | fgrep -v fgrep | ";
  open (DBF, $dbcmd) or die "Cant run $dbcmd\n" ;
  # MAIN LOOP
  while ( <DBF> ) {
   ($f1,$f2,$f3,$f4,$f5,$f6,$f7,$proc,$tmp_dbname,$rest)=split;
   if ($proc eq "$pattern") {
       if ($tmp_dbname eq "($target)" ) {
         return 1;
       }
   }
   return 0;
  }
}


#------------------------------------------------------
#------------------------------------------------------
#-- OK DONE WITH SUBS. NOW MAIN PHASE
#------------------------------------------------------
#------------------------------------------------------

&do_opts or pod2usage(1) or die "do_opts failed\n" ;
if ( defined($version) ) { die "$0\t\t\tVersion: $VER\n"; }


#------------------------------------------------------

# Set defaults based on this ordering
$dbname= $thisdb || $ENV{"_IBM_DBMSNAME"} ;
die("env: _IBM_DBMSNAME is not set. fatal\n") if ($dbname eq '' ) ;

# see if overide CFG file ...
if ( defined($cmdfile) ) { $CFGFILE=$cmdfile;}


if (!defined($force) ) {
  if ( ! &db2_is_up ($dbname) ) { 
    die("DBMS: $dbname is not started\n");
  }
}

#open the file that has the params..
if ( -f $CFGFILE ) {
  open (CFGF,$CFGFILE) or die "Cannot open $CFGFILE\n";
} else {
  print("\n\tYou do not have the CFG file in your home directory\n");
  print("\tI am creating one from a backup. Please update it\n\n");
  system("cp $toolsdir/bin.dbms/CFG $homedir/CFG");
  die("Exiting...\n");
}

# ===========================================


#---------------------------------------------------
#  This routine extracts the current config values
#  for the GET DB CFG form of the dbms command
#  This form gets the db specific config values
#  It grabs the 'names' and values
#---------------------------------------------------
#
sub seed_db
{
   my $cmdf,$DBF,$param, $value;

    # the command
    $cmdf="db2 GET DB CFG FOR $dbname |";

    # see if overide CFG file ...
#    if ( defined($cmdfile) ) { $cmdf=$cmdfile;}
    if (defined $dbg)        { print "$cmdf\n";}

    open (DBF, "$cmdf") or die "Cannot open $infile\n" ;
    while ( <DBF> ) {
      if ( defined $dbg ) { print ; }
      #if it doesnt has a left and right parent, skip
      next if ( ! /\(/ );
      if ( /\(([A-Z0-9_]+)\) = (\S*)\s*$/ ) {
        ($param,$value) = ($1, $2);
        $param=uc($param);
        $db{$param}=$value;
        $dbtype{$param}="DB";
      }
      if ( defined $dbg ) { print "seed_db: $param = $value\n"; }
    }
    close(DBF);
}

#---------------------------------------------------
#  This routine extracts the 'help' list of 
#  config names. We use this to match later.
#---------------------------------------------------
#
sub seed_db_unknown
{
   my @plist, $cmdf,$DBF,$param, $value, $key;

   $cmdf="db2 ? UPDATE DATABASE CONFIGURATION |";
   # see if overide CFG file ...
#   if ( defined($cmdfile) ) { $cmdf=$cmdfile;}

   open (DBF, "$cmdf") or die "Cannot open $infile\n" ;
   while ( <DBF> ) {
      if ( $getem eq "true" ) {
       $_ =~s/,//g; # strip commas
       $_ =~s/\.$//g; # strip final dot
       $_ =~ s/\s+$//; # strip trailing whitespace
       @tmpl = split ;
       push (@plist ,@tmpl);
      }
      if ( /^$/ ) { $getem="false";  }
      if ( /config-keyword:/ ) { $getem="true";  }
   }
   close(DBF);

   # LETS MAKE ONE LIST but these dont have default values...
   foreach $param ( @plist ) {
     $param=uc($param);
     if (! defined($db{$param}) ) {
        if ( defined($dbg) ) { print "seed_db_unknown: help config name $param\n"; }
        $db{$param} = "";
        $dbtype{$param}="DB";
     }
   }
}

#---------------------------------------------------
#  This routine extracts the current config values
#  for the GET DBM CFG form of the dbms command
#  This form gets the enterprise config values
#  It grabs the 'names' and values
#---------------------------------------------------
#
sub seed_dbm
{
    my @plist, $cmdf,$DBF,$param, $value, $key;

    # dumping out current vals (first code)
    $cmdf="db2 GET DBM CFG |";
    # see if overide CFG file ...
    #if ( defined($cmdfile) ) { $cmdf=$cmdfile;}

    if (defined $dbg) { print "$cmdf\n";}
    open (DBF, "$cmdf") or die "Cannot open $infile\n" ;
    while ( <DBF> ) {
      if ( defined $dbg ) { print ; }
      #if it doesnt has a parent, skip
      next if ( ! /\(/ );
      if ( /\(([A-Z0-9_]+)\) = (\S*)\s*$/ ) {
        ($param,$value) = ($1, $2);
        $param=uc($param);
        $db{$param}=$value;
        $dbtype{$param}="DBM";
        if ( defined($dbg) ) { print "seed_dbm: \`$param\` = \`$value\`\n"; }
      }
      #if ( defined $dbg ) { print "$param :: $value\n"; }
    }
    close(DBF);
}


#---------------------------------------------------
#  This routine extracts the 'help' list of 
#  config names. We use this to match later.
#---------------------------------------------------
#
sub seed_dbm_unknown
{
    my @plist, $cmdf,$DBF,$param, $value, $key;

    # second is pulling it from help definition (2nd code)

    $cmdf="db2 ? UPDATE DATABASE MANAGER CONFIGURATION |";
    open (DBF, "$cmdf") or die "Cannot open $infile\n" ;
    while ( <DBF> ) {
      if ( $getem eq "true" ) {
       $_ =~s/,//g; # strip commas
       $_ =~ s/\s+$//; # strip trailing whitespace
       @tmpl = split ;
       push (@plist ,@tmpl);
      }
      if ( /^$/ ) { $getem="false";  }
      if ( /config-keyword:/ ) { $getem="true";  }
    }
    close(DBF);

   # LETS MAKE ONE LIST but these dont have default values...
   foreach $param ( @plist ) {
     $param=uc($param);
     if (! defined($db{$param}) ) {
        if ( defined($dbg) ) { print "seed_dbm_unknown: help config name $param\n"; }
        $db{$param} = "";
        $dbtype{$param}="DBM";
     }
   }
}

sub seed_db2set_unknown
{
    my $cmdf,$DBF,$key;
    $cmdf="db2set -lr |";

    open (DBF, "$cmdf") or die "Cannot open $infile\n" ;
    while ( <DBF> ) {
       chomp;
       $param = uc($_);
       $db{$param} = "";
       $dbtype{$param}="DB2SET";
    }
    close(DBF);
}

sub seed_db2set
{
    my $cmdf,$DBF,$param, $value, $key;

    # one is with dumping out current vals (first code)
    $cmdf="db2set -all | ";
    open (DBF, "$cmdf") or die "Cannot open $infile\n" ;
    while ( <DBF> ) {
      ($scope,$paramline)=split;
      ($param,$value)=split('=',$paramline);
      $param=uc($param);
      $db{$param}=$value;
      $dbtype{$param}="DB2SET";
    }
    close(DBF);
    
}


# Main Program


&seed_db;
&seed_db_unknown;
&seed_dbm;
&seed_dbm_unknown;
&seed_db2set_unknown;
&seed_db2set;


#++++++++++++++++++++++++++++++++++++++++

 if ( defined($dbg) ) {
  foreach $tmp2 ( keys %db     ) { print "DB:  $tmp2\n"; }
 }

#++++++++++++++++++++++++++++++++++++++++
  


# READ THE CHANGE FILE
while ( <CFGF> ) {
  next if ( /#/ ); #skip comments
  next if ( /^[ \t]*$/ ); #skip blank lines
  $_ =~ s/\s+$//; # strip trailing whitespace
  ($param,$value) = split;
  $param=uc($param);
  if ( defined $chg{$param} ) { print "$param is a DUPLICATE\n"; next }
  $chg{$param}=$value;
}
close(CFGF);

# ERROR CHECKING AND PUT ON RIGHT COMMANDLIST
foreach $key ( keys %chg ) {
  if ( !defined( $db{$key}) ) { 
    print "$key is not a valid parameter\n"; 
    next;
  }
  if ( defined($db{$key}) ) {
    if ( uc($db{$key}) eq uc($chg{$key}) ) {
      if ( defined $dbg ) { print "INFO:$key=$chg{$key}, same as default. Ignoring\n"; }
      if ( ! defined($andflt)) { next; }
    }
    if ( defined $dbg ) { print "DB: $key :: $chg{$key} ::\n"; }
    $do_list_db{$key} = $chg{$key};
  }
}



$ts=`date +%m%d%H%M`;
chomp $ts;
$tfile="db2cfg.$ts";
$tmpf="/tmp/$tfile";
open (OUTF,"> $tmpf") or die "Cannot create $tmpf\n";

print OUTF "\n\n";
print OUTF "#######################################################\n";
print OUTF "#######################################################\n";
print OUTF "#DOING FOR DB\n\n";
foreach $key ( keys %do_list_db ) {
  print OUTF "\t\t# Changing $key value from \"$db{$key}\"\n";
  if ( $dbtype{$key} eq "DB" ) {
    print OUTF "db2 UPDATE DB CFG for $dbname using $key \"$do_list_db{$key}\"\n";
  }
  if ( $dbtype{$key} eq "DBM" ) {
    print OUTF "db2 UPDATE DBM CFG using $key \"$do_list_db{$key}\"\n";
  }
  if ( $dbtype{$key} eq "DB2SET" ) {
    print OUTF "db2set $key=\"$do_list_db{$key}\"\n";
  }
}


system("cat $tmpf");

if ( ! defined($yes_go)) {
  print "\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\n";
  print "WARNING: CHANGES based on values in $CFGFILE\n";
  print "Shall we execute the db2 script\? [y|n]: ";
  $response=<>;
  if ( $response ne "y\n" ) {
   print "Aborting ....\n";
   exit;
  }
}

if ( ! defined($comment) ) {
  print "\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\n";
  print "Description: Why we are changing params (one line) \?\n";
  $desc=<>;
} else {
  $desc=$comment;
}

open (CMDFH, "$tmpf" ) or die "Cannot open $tmpf\n";
if (defined($noexec)) {
  # just print results
  while ( <CMDFH> ) { print ; }
} else {
  $good=0; $warn=0; $err=0;
  while ( <CMDFH> ) {
    next if ( /#/ );
    next if ( /^$/ );
    #$ol .= $_, next if ( ! /db2 UPDATE DB/ );
    $cmd=$_;
    $res=`$cmd`; $rc=$?;
     #print "RC: $rc\n";
     if ( $rc == 0 ) { $good++; }
     if ( $rc == 512 ) {
       if ( ( $res =~ /SQL1482W/ ) || 
            ( $res =~ /SQL1362W/ ) ||
            ( $res =~ /DB21026I/ ) ||
            ( $res =~ /SQL1363W/ ) ) { 
          $warn++;
       } else {
          $rc=1024; # unknown treat as if servere
       }
     }
     if ( $rc > 512 ) {
       #print "$ol";
       print "$cmd";
       print "$res";
       $err++;
     }
     $ol="";
  }
  close(CMDFH);

  print "\n";
  print "Successful changes                : $good\n";
  print "OK Warnings (probably bounce msgs : $warn\n";
  print "Errors                            : $err\n";
  print "File of targeted changes:         : $tmpf\n";
  print "\n";


  if ( -d "./configs" ) {
    print "cp $tmpf ./configs/$tfile\n";
    `echo "\n#DESC: $desc\n\n" > ./configs/$tfile` ;
    `cat $tmpf >> ./configs/$tfile` ;
  }
}









#===========================================================
#EVERYTHING after __END__ is ignored by perl, below is the
#help info used by pod_usage
#===========================================================
__END__

=head1 NAME

getcfg.pl - Determines and makes mods to db2 environment

=head1 SYNOPSIS

getcfg.pl [-help|?] [-man] [-noexec] [-andflt] [-f CFG_file]
            [-d dbname] [-y] [ -c "commentdesc" ]

 Options:
   -help            brief help message
   -man             full documentation
   -noexec          Do not execute the UPDATE DB commands. Just show work
   -d               Uses this as the dbms name of the database
   -force           Force running if the dbms is not up
   -version         Displays the version number
   -andflt          Changes match current values are redone
   -f CFG_file      Dont use default ./CFG, use this file instead
   -c descrption    Quoted description of the reason for the change
   -y               Turns off prompt for changing configuration

=head1 DESCRIPTION

B<This program> operates on DB2. It reads <Parameter, Value> pairs
from the file ./CFG, and does some legwork to see if we should change
any of the values. It does some jazzy error checking and only
runs the valid changes.

For each parameter, it determines if it is a parameter of the
specific database or of the entire instance. It applies the 
appropriate DB UPDATE form, to affect the change.

By Default, it uses the file ./CFG, it looks for the environment
variable: DBNAME, These can be overridden.

The -noexec shows u what will execute prior to executing
The -andflt include all parameters that are currently correct, otherwise
it ignores them.

The -y option along with the -c option allows this program to be executed
non-interatively.

=cut  
