#!/usr/bin/perl

###################################################################
#
#   (C) Copyright IBM Corp. 2003 All rights reserved.
#
# This is an IBM Internal Tool developed to aid in managing
# and examining Siebel Applications.  There is no official IBM
# support.  Use at your own risk. Developed for Siebel 7.7 and v8.0
#
###################################################################

use Pod::Usage;
use Getopt::Long;


###################################################
# Options handling
###################################################
sub do_opts {
  my $man = 0, $help = 0;
  use Getopt::Long;
  $retval=GetOptions( 
        "m"       =>\$maj,           
        "i"       =>\$min,           
        "s1"     =>\$submin1,       
        "s2"     =>\$submin2,       
        "a"       =>\$all,           
        'help|?'  =>\$help,          
        man       =>\$man);          

  # handles unknown options. (ARGV isnt empty!)
  if ( $ARGV[0] ) {
    print "Unprocessed by Getopt::Long\n";
    foreach (@ARGV) { print "$_\n"; }
    pod2usage(-message => "Unprocessed by Getopt::Long",
              -exitval => 2, -verbose => 0, -output  => \*STDERR);
  }

  ## Parse options and print usage if there is a syntax error,
  ## or if usage was explicitly requested.
  pod2usage(1)             if $help;
  pod2usage(-verbose => 2) if $man;

  return $retval;
}

&do_opts or pod2usage(1) or die "do_opts failed\n" ;





$default="true";

open DL, "db2level |" || die "Cannot run db2level\n";



while ( <DL> ) {
  if ( /formational tokens .*"DB2 v(.*)",/ ) { 
    ($major,$minor,$subminor1,$subminor2) = split (/\./,$1);
    ($subminor2,$rest) = split (/"/,$subminor2);
     @outl=($major,$minor,$subminor1,$subminor2);
  }
}

  
  if (defined($maj) || defined($min) || defined($submin1) || defined($submin2)) {
   # ok. they used switches, so no default. and we only print what they want
   @outl=();
  }

  if (defined $maj) {
     $default="false";
     push @outl,$major ;
  } 
  if (defined $min) {
     $default="false";
     push @outl,$minor ;
  } 
  if (defined $submin1 ) {
     $default="false";
     push @outl,$subminor1 ;
  }
  if (defined $submin2 ) {
     $default="false";
     push @outl,$subminor2 ;
  }

  $val = shift @outl ;
   if ($default eq "true") {
     print "DB2 Version: ";
   }


  print "$val";
  foreach $val ( @outl ) {
   if ($default eq "false") {
     print " $val";
   } else {
     print ".$val";
   }
  }
  print "\n";


#===========================================================
#EVERYTHING after __END__ is ignored by perl, below is the
#help info used by pod_usage
#===========================================================
__END__

=head1 NAME

db2lvl.pl - calls db2level db2 command and parses it. makes pretty.

=head1 SYNOPSIS

db2lvl.pl 
      [-help|?] 
      [-man] 
      [-m]
      [-i]
      [-s1]
      [-s2]

 Options:
   -help            brief help message
   -man             full documentation
   -m               Print out the major version number (7 out of 7.1.2.3)
   -i               Print out the minor version number (1 out of 7.1.2.3)
   -s1              Print out the subminor1 number (2 out of 7.1.2.3)
   -s2              Print out the subminor2 number (3 out of 7.1.2.3)


=head1 DESCRIPTION

B<This program> man page TODO


=cut  


