#!/usr/bin/perl

###################################################################
#
#   (C) Copyright IBM Corp. 2003 All rights reserved.
#
# This is an IBM Internal Tool developed to aid in managing
# and examining Siebel Applications.  There is no official IBM
# support.  Use at your own risk. Developed for Siebel 7.7 and v8.0
#
###################################################################


use Getopt::Long;
use    Pod::Usage;


$VER=1.3;
######################################################
# Description:
#   This program reads the output of:
#      db2 list tablespaces show detail
#
# HISTORY
# Vers:   Date:         Description
# v1.1    Jun   4 2004  Original version
# v1.2    Jun  14 2004  Making changes. massive
# v1.3    Jul  09 2004  adding -c -y switches
# v1.4    Feb  24 2006  Modified to HOME dir
######################################################


# Perl special variable for OS
$NTID="Windows_NT";
$os=$^O;

# Look for CFG in local dir
$toolsdir=$ENV{'HOME'};
if (!defined($toolsdir)) {
  die("Broken, environment var: _IBM_TOOLSDIR is not defined\n");
}
$CFGFILE   = "$toolsdir/CFG";


#------------------------------------------------------
# Get the OPTIONS
#------------------------------------------------------
sub do_opts {
  my $man = 0, $help = 0;

  use Getopt::Long;
  $retval=GetOptions( 
        "f=s"     =>\$cmdfile,
        "d=s"     =>\$thisdb,
        "noexec"  =>\$noexec,
        "force"   =>\$force,
        "andflt"  =>\$andflt,
        "c=s"     =>\$comment,
        "y"       =>\$yes_go,

        "v"       =>\$version,       # Show Version
        "dbg"     =>\$dbg,           # Debug info printed too
        'help|?'  =>\$help,          # Show Help
        'man'     =>\$man);          # Show Man page

  # handles unknown options. (ARGV isnt empty!)
  if ( $ARGV[0] ) {
    print "Unprocessed by Getopt::Long\n";
    foreach (@ARGV) { print "$_\n"; }
    pod2usage(-message => "Unprocessed by Getopt::Long",
              -exitval => 2, -verbose => 0, -output  => \*STDERR);
  }

    ## Parse options and print usage if there is a syntax error,
    ## or if usage was explicitly requested.
  pod2usage(1)             if $help;
  pod2usage(-verbose => 2) if $man;

  # ## If no arguments were given, then allow STDIN to be used only
  # ## if it's not connected to a terminal (otherwise print usage)
  #   pod2usage("$0: No files given.")  if ((@ARGV == 0) && (-t STDIN));

  return $retval;
}
#----------------------------------------------------------------
#-- end of do_opts  ---------------------------------------------
#----------------------------------------------------------------

########################################################################
# Usage: &db2_is_up("DBMSNAME")
#   Returns 1 if true, the dbms is up and running
#   Returns 0 if false, the dbms is NOT UP.
#
# EXAMPLE:
#   $dbms="SPERF64";
#   if ( &db2_is_up ($dbms) ) { 
#     print("DBMS: $dbms is running\n");
#   } else {
#     print("DBMS: $dbms is not started\n");
#   }
########################################################################
  
sub db2_is_up {
  local $tmp_dbname;
  $USER=$ENV{'USER'};
  $target=$_[0];

  die ("db2_is_up requires the dbms name as the argument\n")
     if ( !defined($target) ) ;
  
  #$pattern="db2sysc";
  $pattern="db2loggw";
  
  if ($USER eq '') { 
    die ("environment variable USER does not exist.\n");
  }
  
  $dbcmd="ps -fu $USER | fgrep $pattern | fgrep -v fgrep | ";
  open (DBF, $dbcmd) or die "Cant run $dbcmd\n" ;
  # MAIN LOOP
  while ( <DBF> ) {
   ($f1,$f2,$f3,$f4,$f5,$f6,$f7,$proc,$tmp_dbname,$rest)=split;
   if ($proc eq "$pattern") {
       if ($tmp_dbname eq "($target)" ) {
         return 1;
       }
   }
   return 0;
  }
}


#------------------------------------------------------
#------------------------------------------------------
#-- OK DONE WITH SUBS. NOW MAIN PHASE
#------------------------------------------------------
#------------------------------------------------------

&do_opts or pod2usage(1) or die "do_opts failed\n" ;
if ( defined($version) ) { die "$0\t\t\tVersion: $VER\n"; }


#------------------------------------------------------

# Set defaults based on this ordering
$dbname= $thisdb || $ENV{"_IBM_DBMSNAME"} ;
die("env: _IBM_DBMSNAME is not set. fatal\n") if ($dbname eq '' ) ;


if (!defined($force) ) {
  if ( ! &db2_is_up ($dbname) ) { 
    die("DBMS: $dbname is not started\n");
  }
}

# ===========================================
sub seed_dbmcfg
{
    my @plist, $cmdf,$DBF,$param, $value, $key;

    # dumping out current vals (first code)
    $cmdf="db2 GET DBM CFG |";
    # see if overide CFG file ...
    if ( defined($cmdfile) ) { $cmdf=$cmdfile;}

    if (defined $dbg) { print "$cmdf\n";}
    open (DBF, "$cmdf") or die "Cannot open $infile\n" ;
    while ( <DBF> ) {
      if ( defined $dbg ) { print ; }
      #if it doesnt has a parent, skip
      next if ( ! /\(/ );
      if ( /\(([A-Z0-9_]+)\) = (.*)\w*$/ ) {
         $param=uc($param);
        ($param,$value) = ($1, $2);
        $dbm{$param}=$value;
      }
      if ( defined $dbg ) { print "$param :: $value\n"; }
    }
    close(DBF);
}


sub seed_unknown
{
    my @plist, $cmdf,$DBF,$param, $value, $key;

    # second is pulling it from help definition (2nd code)

    $cmdf="db2 ? UPDATE DATABASE MANAGER CONFIGURATION |";
    open (DBF, "$cmdf") or die "Cannot open $infile\n" ;
    while ( <DBF> ) {
      if ( $getem eq "true" ) {
       $_ =~s/,//g; # strip commas
       $_ =~ s/\s+$//; # strip trailing whitespace
       @tmpl = split ;
       push (@plist ,@tmpl);
      }
      if ( /^$/ ) { $getem="false";  }
      if ( /config-keyword:/ ) { $getem="true";  }
    }
    close(DBF);

   # LETS MAKE ONE LIST but these dont have default values...
   foreach $param ( @plist ) {
     if (! defined($dbm{$param}) ) {
        if ( defined($dbg) ) { print "ADDING $param to dbm ..\n"; }
        $dbm{$param} = "";
     }
   }
}
&seed_dbmcfg;
&seed_unknown;

foreach $key (keys %dbm) {
  print("$key\t$dbm{$key}\n");
}




#===========================================================
#EVERYTHING after __END__ is ignored by perl, below is the
#help info used by pod_usage
#===========================================================
__END__

=head1 NAME

getcfg.pl - Determines and makes mods to db2 environment

=head1 SYNOPSIS

getcfg.pl [-help|?] [-man] [-noexec] [-andflt] [-f CFG_file]
            [-d dbname] [-y] [ -c "commentdesc" ]

 Options:
   -help            brief help message
   -man             full documentation
   -noexec          Do not execute the UPDATE DB commands. Just show work
   -d               Uses this as the dbms name of the database
   -force           Force running if the dbms is not up
   -version         Displays the version number
   -andflt          Changes match current values are redone
   -f CFG_file      Dont use default ./CFG, use this file instead
   -c descrption    Quoted description of the reason for the change
   -y               Turns off prompt for changing configuration

=head1 DESCRIPTION

B<This program> operates on DB2. It reads <Parameter, Value> pairs
from the file ./CFG, and does some legwork to see if we should change
any of the values. It does some jazzy error checking and only
runs the valid changes.

For each parameter, it determines if it is a parameter of the
specific database or of the entire instance. It applies the 
appropriate DB UPDATE form, to affect the change.

By Default, it uses the file ./CFG, it looks for the environment
variable: DBNAME, These can be overridden.

The -noexec shows u what will execute prior to executing
The -andflt include all parameters that are currently correct, otherwise
it ignores them.

The -y option along with the -c option allows this program to be executed
non-interatively.

=cut  
