#!/usr/bin/perl

###################################################################
#
#   (C) Copyright IBM Corp. 2003 All rights reserved.
#
# This is an IBM Internal Tool developed to aid in managing
# and examining Siebel Applications.  There is no official IBM
# support.  Use at your own risk. Developed for Siebel 7.7 and v8.0
#
###################################################################


use lib qw(./pmods);
use Getopt::Long;
use    Pod::Usage;
use   Misc::Oratab;

###########################################################################
# HISTORY
# Vers:     # Date:     Description
# =======   # =======   ===========================================
# $VV=1.00; # 8/29/06   Original version
  $VV=1.01; # 8/29/06   My changes
#########################################################################

${_IBM_STATSVER}=$VV;
$VER=${_IBM_STATSVER};
undef($VV);

$sid="";
$homedir="";

#########################################################################
#-----------------------------------------------------------------------#
#-------------        Supporting Subroutines   -------------------------#
#-----------------------------------------------------------------------#
#########################################################################

sub do_opts {
  my $man= 0, $help= 0;

  use Getopt::Long;
  $retval=GetOptions( 
          "sid=s"=>\$sid,
      "homedir=s"=>\$homedir,
        "sidonly"=>\$sidonly,
       "homeonly"=>\$homeonly,
     "force_menu"=>\$force_menu,
      "outfile=s"=>\$outfile,
            "dbg"=>\$dbg,
              "v"=>\$vershun,
         'help|?'=>\$help,
              man=>\$man
            ) ;

  # handles unknown options. (ARGV isnt empty!)
  if ( $ARGV[0] ) {
    print "Unprocessed by Getopt::Long\n";
    foreach (@ARGV) { print "$_\n"; }
    pod2usage(-message=> "Unprocessed by Getopt::Long",
              -exitval=> 2, -verbose=> 0, -output=> \*STDERR);
  }

    ## Parse options and print usage if there is a syntax error,
    ## or if usage was explicitly requested.
  pod2usage(1)             if $help;
  pod2usage(-verbose=> 2) if $man;

  # ## If no arguments were given, then allow STDIN to be used only
  # ## if it's not connected to a terminal (otherwise print usage)
  #   pod2usage("$0: No files given.")  if ((@ARGV== 0) && (-t STDIN));

  return $retval;
}
&do_opts or pod2usage(1) or die "do_opts failed\n" ;
if (defined($vershun)) { die("\t $0: Version: v$VER\n");}



#########################################################################
#-----------------------------------------------------------------------#
#-------------        Main Program             -------------------------#
#-----------------------------------------------------------------------#
#########################################################################


$options='';

if (defined($force_menu)) { $options="force_menu"; }
if (defined($dbg)) {print("FORCE: $options\n"); }



$out=&get_ora_variables($sid,$homedir,$options);
($sid,$homedir)=split(' ',$out);

if (defined($sidonly))  { print("$sid\n");     exit; }
if (defined($homeonly)) { print("$homedir\n"); exit; }

if (defined($dbg)) {print("sid: $sid home: $home out: $out\n");}

if (defined($outfile)) {
  open(OUTF,"> $outfile") or die("Abort: Cannot write $outfile\n");
  print OUTF "export ORACLE_HOME=$homedir\n";
  print OUTF "export ORACLE_SID=$sid\n";
  close(OUTF);
} else {
  print("$sid $homedir\n");
}


#EVERYTHING after __END__ is ignored by perl, below is the
#help info used by pod_usage
#===========================================================
__END__

=head1 NAME

oratab.pl - Returns valid oracle instance and home directories from the /etc/oratab file.

=head1 SYNOPSIS

oratab.pl [-help|?] [-man] [-sidonly] [-homeonly] [-sid valid_sid_value] [-homedir valid_oracle_homedir]

 Options:
   -help            brief help message
   -man             full documentation
   -sidonly         Print only the sid value returned
   -homeonly        Print only the oracle home directory
   -sid             Provide the oracle sid for lookup
   -homedir         Provide the oracle home directory for lookup

=head1 DESCRIPTION

oratab.pl reads the oracle /etc/oratab file and provide a menu selection
for choosing oracle instances. Also if given an oracle sid and home
directory, will validate that the values are within the /etc/oratab file.
This can be useful for choosing oracle instances within a .profile or
dynamically to change the values within the environment.

oratabl.pl has options for getting any combination of values based
on input switches and input values. If there isnt enough information
to uniquely determine the output values, a menu is given of the possible
choices for selection.

-sid     option provides a means to setting the oracle instance sid.

-homedir option provides a means to setting the oracle home directory.

Two options affect the output. If -sidonly is given, only the sid value
is printed. If -homeonly option is given, the oracle home directory is
printed.  Otherwise both are printed in 'sid homedir' order.


=cut  

