#!/bin/ksh

###################################################################
#
#   (C) Copyright IBM Corp. 2003 All rights reserved.
#
# This is an IBM Internal Tool developed to aid in managing
# and examining Siebel Applications.  There is no official IBM
# support.  Use at your own risk. Developed for Siebel 7.7 and v8.0
#
###################################################################
export HOME=${HOME?}



#*****************************************************************#
#                                                                 #
#                   Function Definitions                          #
#                                                                 #
#*****************************************************************#

# USAGE STMT
usage(){
    print startdb - Starts up the oracle database.
	print "Usage: startdb [-ioldh]"
	print "  -i   Dont bring up the isqlsplus service"
	print "  -o   Dont bring up the oem service"
	print "  -l   Dont bring up the listener service"
	print "  -d   Dont bring up the database service"
	print "  -h   Usage help"
}



isdbrunning() {
  #set -x
  check_stat=`ps -ef|grep ${ORACLE_SID}|grep pmon|wc -l`;
  oracle_num=`expr $check_stat`
  if [ $oracle_num -lt 1 ]
  then
    #echo Nope DBMS is not up and running
    return 16
  fi
  

  rm -f /tmp/check_$ORACLE_SID.ora
  #*************************************************************
  # Test to see if Oracle is accepting connections
  #*************************************************************
  $ORACLE_HOME/bin/sqlplus -s "/ as sysdba"  > /tmp/check_$ORACLE_SID.ora << EOF
  select * from v\$database;
  exit
EOF
  check_stat=`cat /tmp/check_$ORACLE_SID.ora | grep -i error | wc -l`;
  oracle_num=`expr $check_stat`
  if [ $oracle_num -ne 0 ]
  then
    #echo dbms is not taking connections.
    return 8
  else
   #echo dbms is up
   return 0
  fi
} 

islsnrunning() {
  #set -x
  check_stat=`ps -ef|grep ${USER}|grep tnslsnr | fgrep -v grep |wc -l`;
  oracle_num=`expr $check_stat`
  if [ $oracle_num -lt 1 ]
  then
    #echo Nope DBMS is not up and running
    return 16
  fi
} 

isemagentrunning() {
  #set -x
  check_stat=`ps -ef|grep ${USER}|grep emagent | fgrep -v grep |wc -l`;
  oracle_num=`expr $check_stat`
  if [ $oracle_num -lt 1 ]
  then
    #echo Nope DBMS is not up and running
    return 16
  fi
} 

isisqlplusrunning() {
  #set -x
  httpfile="$ORACLE_HOME/oc4j/j2ee/isqlplus/config/http-web-site.xml" 
  if [ -f "$httpfile" ] ; then
    portnum=`cat "$httpfile" | fgrep -- "web-site port" | \
	sed -e "/.*port=\"/s///" -e "/\" display.*$/s///"`
    portchk=`netstat -an | fgrep LISTEN | fgrep ".$portnum " |wc -l`
    oracle_num=`expr $portchk`
    echo $oracle_num
    if [ $oracle_num -lt 1 ] ; then
       #echo Nope DBMS is not up and running
       return 16
    fi
  fi
} 

start_listener() {
  #set -x
  if [ "$nolsnr" != "" ] ; then
    comp="listener"
    print "\nBYPASSING $comp \t... Option to NOT start $comp service given;"
    return;
  else
    islsnrunning
    if [ $? != 16 ] ; then
       echo "Listener is already Running ..."
       return
    fi
    lsnrctl start > $HOME/.ibmtools/oracle/logs/lsnrstart.out 2>&1
    if [ $? == 0 ] ; then
      echo "Listener has been started ..."
    else
      echo "Listener is already Running ..."
    fi
  
    echo "Listener start output: $HOME/.ibmtools/oracle/logs/lsnrstart.out"
    echo
    sleep 1
  fi
}

start_emagent() {
  #set -x
  if [ "$noemctl" != "" ] ; then
    comp="OEM"
    print "\nBYPASSING $comp \t\t... Option to NOT start $comp service given;"
  else
    isemagentrunning
    if [ $? != 16 ] ; then
       echo "emctl is already Running ..."
       return
    fi

    echo "running emctl start dbconsole ..."
    emctl start dbconsole
    if [ $? == 0 ] ; then
      echo "emctl has been started ..."
    else
      echo "emctl is already Running ..."
    fi
  
    echo "emctl start output: $HOME/.ibmtools/oracle/logs/emctl.out"
    echo
    sleep 1
  fi
}

start_isqlplus() {

  #set -x
  comp="isqlplus"
  release=`echo ${_IBM_ORAVERS} | perl -lane  '@F=split(/\./); print("$F[0]")'`;
  if (( release > 10 ))  then
    print "\nBYPASSING $comp \t\t... De-supported in this version;"
    return;
  fi

  if [ "$noisqlplus" != "" ] ; then
    print "\nBYPASSING $comp \t... Option to NOT start $comp service given;"
  else
    isisqlplusrunning
    if [ $? != 16 ] ; then
       echo "isqlplus is already Running ..."
       return
    fi
    isqlplusctl start
    if [ $? == 0 ] ; then
      echo "isqlplus has been started ..."
    else
      echo "isqlplus is already Running ..."
    fi
  
    echo "isqlplus start output: $HOME/.ibmtools/oracle/logs/isqlplus.out"
    echo
    sleep 1
  fi
}

start_dbms() {
  #set -x
  if [ "$nodbms" != "" ] ; then
    comp="dbms"
    print "\nBYPASSING $comp \t\t... Option to NOT stop $comp service given;"
    return 0;
  else
    isdbrunning
    DBSTATE=$?
    if [ $DBSTATE != 16 ] ; then
      case $DBSTATE in
         0) echo "ERROR:   DBMS is already running"                             ;;
         8) echo "ERROR:   DBMS is already running and NOT taking connections"  ;;
        16) echo "DBMS is NOT running"                                        ;;
         *) echo "UNKNOWN VALUE $?" ;      exit 2                             ;;
        esac
        echo "Aborting ..."
        return $DBSTATE
    fi
  
    echo "\nStarting Oracle Database ..."
    echo
    sqlplus -S "/ as sysdba" <<EOF
    startup
    quit
EOF
    return 0;
  fi
}

#*****************************************************************#
#                                                                 #
#                           Main Program                          #
#                                                                 #
#*****************************************************************#

#noisqlplus="true"
#noemctl="true"
#nolsnr="true"
#nodbms="true"

while getopts "oildh" arg
do
  case $arg in
    i) noisqlplus="true" ;;
    o) noemctl="true" ;;
    l) nolsnr="true" ;;
    d) nodbms="true" ;;
    h|*) usage
    	exit 1 ;;
  esac
done
shift $(($OPTIND - 1))

###########################################
# SHALL WE START THE LISTENER?
###########################################
#

  start_listener;


###########################################
# OK START UP THE DBMS ..
###########################################
#

  start_dbms;


###########################################
# SHALL WE START THE isqlplusctl? FOR OEM?
###########################################
#

  start_isqlplus;

###########################################
# SHALL WE START THE EMCTL FOR OEM?
###########################################
#

  start_emagent;

