#!/bin/ksh

###################################################################
#
#   (C) Copyright IBM Corp. 2003 All rights reserved.
#
# This is an IBM Internal Tool developed to aid in managing
# and examining Siebel Applications.  There is no official IBM
# support.  Use at your own risk. Developed for Siebel 7.7 and v8.0
#
###################################################################
export HOME=${HOME?}



#*****************************************************************#
#                                                                 #
#                   Function Definitions                          #
#                                                                 #
#*****************************************************************#

# USAGE STMT
usage(){
	print stopdb - Stops up the oracle database.
	print "Usage: stopdb [-ioldh]"
	print "  -i   Dont bring down the isqlsplus service"
	print "  -o   Dont bring down the oem service"
	print "  -l   Dont bring down the listener service"
	print "  -d   Dont bring down the database service"
	print "  -h   Usage help"
}

isdbrunning() {
  #set -x
  check_stat=`ps -ef|grep ${ORACLE_SID}|grep pmon|wc -l`;
  oracle_num=`expr $check_stat`
  if [ $oracle_num -lt 1 ]
  then
    #echo Nope DBMS is not up and running
    return 16
  fi
  

  rm -f /tmp/check_$ORACLE_SID.ora
  #*************************************************************
  # Test to see if Oracle is accepting connections
  #*************************************************************
  $ORACLE_HOME/bin/sqlplus -s "/ as sysdba"  > /tmp/check_$ORACLE_SID.ora << EOF
  select * from v\$database;
  exit
EOF
  check_stat=`cat /tmp/check_$ORACLE_SID.ora | grep -i error | wc -l`;
  oracle_num=`expr $check_stat`
  if [ $oracle_num -ne 0 ]
  then
    #echo dbms is not taking connections.
    return 8
  else
   #echo dbms is up
   return 0
  fi
} 

islsnrunning() {
  #set -x
  check_stat=`ps -ef|grep ${USER}|grep tnslsnr | fgrep -v grep |wc -l`;
  oracle_num=`expr $check_stat`
  if [ $oracle_num -lt 1 ]
  then
    #echo Nope DBMS is not up and running
    return 16
  fi
} 

isemagentrunning() {
  #set -x
  check_stat=`ps -ef|grep ${USER}|grep emagent | fgrep -v grep |wc -l`;
  oracle_num=`expr $check_stat`
  if [ $oracle_num -lt 1 ]
  then
    #echo Nope DBMS is not up and running
    return 16
  fi
} 

isisqlplusrunning() {
  #set -x
  httpfile="$ORACLE_HOME/oc4j/j2ee/isqlplus/config/http-web-site.xml" 
  if [ -f "$httpfile" ] ; then
    portnum=`cat "$httpfile" | fgrep -- "web-site port" | \
	sed -e "/.*port=\"/s///" -e "/\" display.*$/s///"`
    portchk=`netstat -an | fgrep LISTEN | fgrep ".$portnum " |wc -l`
    oracle_num=`expr $portchk`
    if [ $oracle_num -lt 1 ] ; then
       #echo Nope DBMS is not up and running
       return 16
    fi
  fi
} 

stop_listener() {
  #set -x
  if [ "$nolsnr" != "" ] ; then
    comp="listener"
    print "\nBYPASSING $comp \t... Option to NOT stop $comp service given;"
    return;
  else
    islsnrunning
    if [ $? != 0 ] ; then
       echo "Listener is already Stopped ..."
       return
    fi
    lsnrctl stop > $HOME/.ibmtools/oracle/logs/lsnrstop.out 2>&1
    if [ $? == 0 ] ; then
      echo "Listener has been stopped ..."
    else
      echo "Listener is already stopped ..."
    fi
  
    echo "Listener stop output: $HOME/.ibmtools/oracle/logs/lsnrstop.out"
    echo
    sleep 1
  fi
}

stop_emagent() {
  #set -x
  if [ "$noemctl" != "" ] ; then
    comp="OEM"
    print "\nBYPASSING $comp \t\t... Option to NOT stop $comp service given;"
  else
    isemagentrunning
    if [ $? != 0 ] ; then
       echo "emctl is already stoppped ..."
       return
    fi

    echo "running emctl stop dbconsole ..."
    emctl stop dbconsole
    if [ $? == 0 ] ; then
      echo "emctl has been stopped ..."
    else
      echo "emctl is already stopped ..."
    fi
    echo
    sleep 1
  fi
}

stop_isqlplus() {
  #set -x

  comp="isqlplus"

  release=`echo ${_IBM_ORAVERS} | perl -lane  '@F=split(/\./); print("$F[0]")'`;
  if (( release > 10 ))  then
    print "\nBYPASSING $comp \t\t... De-supported in this version;"
    return;
  fi

  if [ "$noisqlplus" != "" ] ; then
    print "\nBYPASSING $comp \t... Option to NOT stop $comp service given;"
  else
    isisqlplusrunning
    if [ $? != 0 ] ; then
       echo "isqlplus is already stopped ..."
       return
    fi
    isqlplusctl stop
    if [ $? == 0 ] ; then
      echo "isqlplus has been stopped ..."
    else
      echo "isqlplus is already stopped ..."
    fi
    echo
    sleep 1
  fi
}

stop_dbms() {
  #set -x
  if [ "$nodbms" != "" ] ; then
    comp="dbms"
    print "\nBYPASSING $comp \t\t... Option to NOT stop $comp service given;"
    return 0;
  else
    isdbrunning
    DBSTATE=$?
    if [ $DBSTATE != 0 ] ; then
      case $DBSTATE in
         8) echo "ERROR:   DBMS is running and NOT taking connections"  ;;
        16) echo "DBMS is already stopped"                              ;;
         *) echo "UNKNOWN VALUE $?" ;      exit 2                       ;;
        esac
        echo "Aborting ..."
        return $DBSTATE
    fi
  
    echo "\nStopping Oracle Database ..."
    echo
    sqlplus -S "/ as sysdba" <<EOF
    shutdown immediate
    quit
EOF
  return 0;
  fi
}

#*****************************************************************#
#                                                                 #
#                           Main Program                          #
#                                                                 #
#*****************************************************************#

#noisqlplus="true"
#noemctl="true"
#nolsnr="true"
#nodbms="true"

# SHELL OPTIONS
while getopts "ioldh" arg
do
  case $arg in
     i) noisqlplus="true" ;;
     o) noemctl="true" ;;
     l) nolsnr="true" ;;
     d) nodbms="true" ;;
     h|*) usage
        exit 1 ;;
    esac
done
shift $(($OPTIND - 1))

###########################################
# SHALL WE START THE EMCTL FOR OEM?
###########################################
#

  stop_emagent;

###########################################
# SHALL WE START THE isqlplusctl? FOR OEM?
###########################################
#

  stop_isqlplus;

###########################################
# OK START UP THE DBMS ..
###########################################
#

  stop_dbms;

###########################################
# SHALL WE START THE LISTENER?
###########################################
#

  stop_listener;

